#! /usr/bin/perl -w
$| = 1;

##
##  Install UCS on your system, i.e. adjust Perl binary to use and library paths.
##  Usage: perl Install.perl
##

## Filename: Install.perl
## Modified: Fri Jul  9 12:31:48 2004 (evert)   
##   Author: Stefan Evert
##  Purpose: "install", i.e. set correct paths in UCS scripts and modules

use Getopt::Long;
use FileHandle;
use File::Glob qw<:glob>;
use FindBin;
use Config;

##
##  Step 1: process command-line switches
##

$Force = 0;
@Includes = ();
$Rbin = undef;
$Rhome = undef;
$Verbose = 0;
$Help = 0;

Getopt::Long::Configure(qw<no_auto_abbrev>);
$ok = GetOptions(
		 "force|f" => \$Force,	        # install even when prerequisites are not satisfied
		 "include|I=s" => \@Includes,	# additional Perl include directories
		 "R=s" => \$Rbin,		# location of the R interpreter program
		 "Rhome|rh=s" => \$Rhome,	# home directory of R distribution (for RSPerl support only)
		 "verbose|v" => \$Verbose,	# print more detailed information (e.g. which files are edited)
		 "help|h" => \$Help,		# short help message
		);
die 
  "\nUsage:  perl Install.perl [options]\n",
  "  -I <dir>   add directory <dir> to Perl module search path\n",
  "    --include=<dir>  [use repeatedly for multiple directories]\n",
  "  -R <file>  fully qualified filename of R interpreter program\n",
  "    --R=<file>       [relative path is not accepted]\n",
  "  -rh <dir>  home directory of R distribution\n",
  "    --Rhome=<dir>    [only needed if \"R RHOME\" doesn't work]\n",
  "  -f         force installation despite missing prerequisites\n",
  "    --force\n",
  "  -v         prints more detailed information (e.g. which files are edited)\n",
  "    --verbose\n",
  "  -h         this help page\n",
  "    --help\n\n"
  if @ARGV or $Help or (not $ok);

##
##  Step 2: check Perl version (5.6.1 or newer)
##          [install script must be run with the Perl interpreter to be used by UCS]
##

# check Perl version (5.6.1 required)
if (not defined eval 'require 5.006_001; 1') {
  chomp($msg = $@);
  $msg =~ s/,.*$//;		# cut off "explanation" after comma
  if ($Force) {
    warn "Perl version error: $msg. Installing anyway.\n";
  }
  else {
    die "Perl version error: $msg. Aborted.\n";
  }
}

$AllowWhitespace = 0;				# path to Perl binary shouldn't contain whitespace
# find Perl interpreter (absolute path for #! line)
$Perl = $Config{"perlpath"};
die "Yikes! Your Perl interpreter ($Perl) doesn't seem to be an executable program. Aborted.\n"
  unless -x $Perl;
die "Error: need absolute path to Perl binary ($Perl). Aborted.\n"
  unless $Perl =~ /^\//;
die_if_funny_chars($Perl);

##
##  Step 3: determine the base directory of the UCS system
##

$AllowWhitespace = 0;				# no whitespace allowed in UCS library path
# determine library directory
$Basedir = $FindBin::RealBin;
die "Couldn't determine UCS base directory ($Basedir is wrong). Aborted.\n"
  unless -d "$Basedir/Perl/lib" and -f "$Basedir/Install.perl";
die "Error: need absolute path to UCS base directory ($Basedir). Aborted.\n"
  unless $Basedir =~ /^\//;
if (has_funny_chars($Basedir)) {
  print STDERR "The UCS system is located in a directory that contains special\n";
  print STDERR "characters (see the error message below).  Please unpack the\n";
  print STDERR "UCS distribution in another location and run Install.perl again.\n\n";
  die_if_funny_chars($Basedir);
}
($UCS_Basedir = $Basedir) =~ s{/System/?}{}	# we need the real UCS base dir for UCS/R under Cygwin
  or die "Error: something wrong with UCS system base directory ($Basedir). Aborted.\n";

##
##  Step 4: activate special support for Win32/Cygwin installation
##

$Cygwin = ($Config{'osname'} =~ /cygwin/i) ? 1 : 0; # experimental support for Win32/Cygwin installation 
if ($Cygwin) {
  print "[activating experimental support for Cygwin environment]\n";

  @lines = split /\n/, `df $Basedir`;		# find Windows FS path to base directory
  die "Internal error: can't parse output of `df $Basedir`\n", "[@lines]\n"
    unless @lines == 2;
  @words = split " ", $lines[1];
  $mountpoint = pop @words;
  die "Error: under Cygwin, the UCS system must be installed in the root filesystem mounted on /\n",
    "[seems to be mounted on $mountpoint]\n"
      unless $mountpoint eq "/";
  $CygwinBase = shift @words; # df should list Windows path to Cygwin base directory as file system
  $CygwinBase =~ tr[\\][/];   # use Unix-style directory separators (works with Windows version of R)
  die "Error: the Cygwin base directory $CygwinBase doesn't seem to be correct, sorry.\n"
    unless $CygwinBase =~ /^[A-Z]:\//;

  $winBasedir = windows_path($Basedir); # test Windows path to UCS base directory
  die "Error: can't access UCS base directory under Windows path $winBasedir\n"
    unless -d "$winBasedir/Perl/lib" and -f "$winBasedir/Install.perl";

  $winRootdir = windows_path("/");     # Windows path to root directory of Cygwin tree
  die "Error: can't find bash.exe (should be \"$winRootdir/bin/bash.exe\". Aborted.\n"
    unless -f "$winRootdir/bin/bash.exe";
}

##
##  Step 5: validate command-line options
##

$AllowWhitespace = 0; # Perl include directories must never contain whitespace
die_if_funny_chars(@Includes);
@errors = map {"-I $_"} grep {not is_absolute_path($_)} @Includes;
die 
  "Error: no relative paths allowed for Perl include directories (-I option)\n",
  "       [@errors]\n"
  if @errors;

$AllowWhitespace = 1; # allows whitespace in file/dir names (may be necessary under Cygwin)
if (defined $Rbin) {
  die_if_funny_chars($Rbin);
  die "Error: $Rbin is not a fully qualified filename (absolute path)\n"
    unless is_absolute_path($Rbin);
  die "Error: $Rbin is not an executable program\n"
    unless -f $Rbin and -x $Rbin;
}

if (defined $Rhome) {
  die_if_funny_chars($Rhome);
  die "Error: $Rhome (the R home directory) must be an absolute path\n"
    unless is_absolute_path($Rhome);
  die "Error: $Rhome (the R home directory) is not a directory\n"
    unless -d $Rhome;
}


##
##  Step 6: check for presence of required and recommended Perl modules
##          [looks also in additional include directories specified by user]
##

# check for required modules
@mods = qw<Expect Pod::Perldoc>;
@missing = grep { not defined eval "use lib \@Includes; use $_; 1" } @mods;
if (@missing > 0) {
  if ($Force) {
    warn "Warning: one or more modules required by UCS are not installed on your system.\n",
      "\t[missing module(s): @missing]\n",
	"Installing anyway (--force).  USE AT YOUR OWN RISK!\n";
  }
  else {
    die "Error: one or more modules required by UCS are not installed on your system.\n",
      "\t[missing module(s): @missing]\n";
  }
}

# check for optional modules (ask user whether to proceed)
@mods = qw<Tk Tk::Pod Term::ReadKey>;		
@missing = grep { not defined eval "use lib \@Includes; use $_; 1"} @mods;
if (@missing > 0) {
  print "Warning: one or more recommended modules are not installed on your system.\n",
    "UCS/Perl works without these modules, but some functions may not be available.\n",
      "\t[missing module(s): @missing]\n";
  if (not $Force) {
    yes_or_no("Continue without these modules?")
      or die "Installation aborted.\n";
  }
}

# check for expected future dependencies (just issue a warning)
@mods = qw<Tk::TableMatrix>;
@missing = grep { not defined eval "use lib \@Includes; use $_; 1" } @mods;
print
  "Notice: the following modules, which may be required by future releases of\n",
  "UCS/Perl, are not installed on your system. Currently, there is no need for\n",
  "these modules, so I will continue with the installation.\n",
  "\t[missing module(s): @missing]\n"
  if @missing > 0;

##
##  Step 7: find R interpreter program and home directory
##          [unless specified by user]
##

$AllowWhitespace = 1;	                  # path to R may contain whitespace
# find R binary in user's path (or the R home directory when specified with --Rhome)
if (not defined $Rbin) {
  $Rname = ($Cygwin) ? "Rterm.exe" : "R"; # in the Windows port, the command-line version is called Rterm.exe
  $Rbin = find_binary($Rname);

  if ((not defined $Rbin) and $Rhome) {
    $Rbin = "$Rhome/bin/$Rname";	  # if R home directory was specified, look for R interpreter there 
    undef $Rbin
      unless -f $Rbin and -x $Rbin;
  }

  if ($Force) {
    if (not defined $Rbin) {
      warn "Warning: I couldn't find the R program in your search path.\n",
	"I will continue with the installation anyway (--force), but the\n",
	  "UCS/R part will not be available and UCS/Perl will not function\n",
	    "reliably.  USE AT YOUR OWN RISK!\n";
      $Rbin = undef;
    }
  }
  else {
    die "Error: Couldn't find R program in your search path. Please make sure\n",
      "that R is installed and specify the fully qualified filename of the R\n",
	"interpreter with the -R option if necessary.\n"
	    unless defined $Rbin;
  }

  die_if_funny_chars($Rbin)
    if defined $Rbin;
}

# check R version (and whether interpreter works)
$output = `'$Rbin' --version 2>&1`;
if ($output =~ /^R \s+ ([0-9]+) \. ([0-9]+ (\.[0-9]+)?) (\s| $ )/mx) {
  $R_major = $1;
  $R_minor = $2;
  $R_version = "$R_major.$R_minor";
  unless ($R_major >= 2 or ($R_major == 1 and $R_minor >= 6)) {
    if ($Force) {
      warn "Warning: R interpreter (version $R_version) is too old (1.6 or newer required)\n",
	"I will continue with the installation anyway (--force), but UCS/R and some\n",
	  "parts of UCS/Perl may not work properly.  USE AT YOUR OWN RISK!\n";
    }
    else {
      die "Error: R interpreter (version $R_version) is too old (1.6 or newer required)\n",
	"Please install a more recent version of R (see \"doc/install.txt\").\n";
    }
  }
}
else {
  $R_version = "???";
  if ($Force) {
    warn "Warning: Can't run the R interpreter ($Rbin).\n",
      "I will continue with the installation anyway (--force), but UCS/R and various\n",
	"parts of UCS/Perl will not work!  USE AT YOUR OWN RISK!\n";
  }
  else {
    die "Error: Can't run the R interpreter ($Rbin).\n",
      "Please make sure that R is installed and specify the fully qualified\n",
	"filename of the R interpreter with -R if necessary (see \"doc/install.txt\").\n";
  }
}

##
##  Step 8: find other helper applications
##

$AllowWhitespace = 0;	                  # no whitespace in a2ps path (because it will be used in a pipe command)
# check for a2ps in user's path (ask user whether to proceed if not found)
@progs = qw<a2ps>;
@missing = grep { not defined find_binary($_) } @progs;
if (@missing > 0) {
  print "Warning: one or more helper applications are not installed on your system.\n",
    "UCS/Perl works without these programs, but some functions may not be available.\n",
      "\t[missing application(s): @missing]\n";
  if (not $Force) {
    yes_or_no("Continue without these helper applications?")
      or die "Installation aborted.\n";
  }
}
$a2ps = find_binary("a2ps");			# may be undefined
die_if_funny_chars($a2ps)
  if defined $a2ps;

##
##  Step 7: print summary of UCS configuration
##

$Libdir = $Basedir."/Perl/lib";
$Bindir = $Basedir."/Perl/bin";
$Tooldir = $Basedir."/Perl/tools";

$Version = join(".", @Config{qw<PERL_REVISION PERL_VERSION PERL_SUBVERSION>});

# print summary of configuration information
print "\n";
print "=== UCS System Configuration ===\n";
print "\n";
print "Perl Version:       $Version\n";
print "R Version:          $R_version\n";
print "Perl Interpreter:   $Perl\n";
print "R Interpreter:      ", (defined $Rbin) ? $Rbin : "-- not installed --", "\n";
print "a2ps program:       ", (defined $a2ps) ? $a2ps : "-- not installed --", "\n";
print "UCS Base Directory: $Basedir\n";
print "\n";
print "================================\n";
print "\n";

##
##  Step 8: update configuration information in UCS.pm, ucs.R, and UCS/Perl scripts
## 

# clean up Backup files (uses bsd_glob() to avoid messing up on whitespace, which isn't allowed anyway)
@Backups = (bsd_glob("$Bindir/*~"), bsd_glob("$Tooldir/*~"));
$N = @Backups;
if ($N > 0) {
  print "Deleting $N emacs backup file(s).\n";
  foreach $file (@Backups) { unlink $file }
}

# assemble shebang line to insert
$shebang = "#!$Perl -w ".join(" ", map {"-I$_"} (@Includes, $Libdir));
print "  [Perl command: $shebang]\n"
  if $Verbose;

# check which files require substitution
@Files = (bsd_glob("$Bindir/*"), bsd_glob("$Tooldir/*"));
$N = @Files;
print "Substituting #! line in $N files ...\n";

# substitute shebang line in Perl scripts
foreach $file (@Files) {
  substitute($file, qr/^\#!.*perl/, $shebang);
  system "chmod", "a+rx", $file; # make sure the files are marked as executables
}

print "Editing configuration files ...\n";
$R_Basedir = quote(windows_path($Basedir));

# substitute base directory in UCS.pm (UCS/Perl configuration)
$signature = ";\t # in-place edit by Install.perl\n";
substitute("$Libdir/UCS.pm", 
	   qr/^\s*\$BaseDir\s*=/ => '$BaseDir = '.quote($Basedir).$signature,
	   qr/^\s*\$PerlCmd\s*=/ => '$PerlCmd = '.quote($shebang).$signature,
	   qr/^\s*\$PerlBin\s*=/ => '$PerlBin = '.quote($Perl).$signature,
	   qr/^\s*\@PerlLibs\s*=/ => '@PerlLibs = ('.join(", ", map {quote($_)} (@Includes, $Libdir)).')'.$signature,
	   qr/^\s*\$Rbin\s*=/ => '$Rbin = '.quote($Rbin).$signature,
	   qr/^\s*\$a2ps\s*=/ => '$a2ps = '.quote($a2ps).$signature,
	   qr/^\s*\$R_BaseDir\s*=/ => "\$R_BaseDir = $R_Basedir".$signature,
	   );

# substitute base directory in ucs.R (UCS/R configuration), using Windows path under Cygwin
$signature = "\t # in-place edit by Install.perl\n";
substitute("$Basedir/R/lib/ucs.R", 
	   qr/^\s*ucs\.basedir\s*<-/ => "ucs.basedir <- $R_Basedir".$signature,
	   # ... and tell UCS/R whether it's running under Cygwin	   
	   qr/^\s*ucs\.cygwin\s*<-/ => "ucs.cygwin <- ".(($Cygwin) ? "TRUE" : "FALSE").$signature,
	   qr/^\s*ucs\.dataset\.dir\s*<-/ => "ucs.dataset.dir <- ".quote(windows_path("$UCS_Basedir/DataSet")).$signature,
	   qr/^\s*ucs\.rootdir\s*<-/ => "ucs.rootdir <- ".quote(windows_path("/")).$signature,
	   );


##
##  Step 9: test installation by running the ucs-config command
##

print "\nTesting installation ...\n";
$result = system "$Bindir/ucs-config";
die "\nOops! Something went wrong -- installation aborted. :o(\n\n"
  if $result != 0;

# now that we know ucs-config works, insert version information into UCS/R config
chomp($version = `$Bindir/ucs-config --version`);
substitute("$Basedir/R/lib/ucs.R", 
	   qr/^\s*ucs\.version\s*<-/ => "ucs.version <- '$version'".$signature,
	  );
if (-f "$Basedir/R/package/DESCRIPTION") {
  substitute("$Basedir/R/package/DESCRIPTION",
	     qr/^\s*Version:/ => "Version: $version\n");
}

# that's all we have to do
print "Installation successful.\n\n";

##
##  ======================================================================
##

## substitute($file, $regex, $line [, $regex2, $line2, ...]);
##   in-place edit $file, substituting every line matching $regex by $line;
##   $file is backed up as "$file~"; dies unless exactly one substitution is made
sub substitute {
  my $file = shift;
  print "  [$file]\n" 
    if $Verbose;
  my $fh = new FileHandle $file;		# read file into array (list of lines)
  die "Can't read '$file'. Aborted.\n"
    unless defined $fh;
  my @lines = <$fh>;
  die "Read error.\n"
    unless @lines >= 6;			        # rough check :o)   [might check against "wc -l" instead]
  $fh->close;
  while (@_) {					# process
    my $rx = shift;
    my $line = shift;
    $line .= "\n"
      unless $line =~ /\n$/;
    die "Internal error: odd number of arguments to substitute()\n"
      unless defined($rx) and defined($line);
    my $substitutions = 0;			# should replace exactly 1 line
    foreach my $l (@lines) {
      if ($l =~ $rx) {
	$l = $line;
	$substitutions++;
      }
    }
    die "\n!! Substitution error ($substitutions line(s) modified). Aborted.\n", 
      "!! FILE: $file\n", "!! LINE: $line\n"
	unless $substitutions == 1;
  }
  my $backup = "$file~";			# back up original file
  unlink $backup
    if -f $backup;
  system "cp", "-p", $file, $backup;		# we'll ignore failure to make a backup copy
  $fh = new FileHandle "> $file";		# write modified lines back to file
  die "Write error.\n"
    unless defined $fh;
  map { $fh->print($_) } @lines;
  $fh->close
    or die "Write error. '$file' may be damaged!\n";
  system "touch", "-r", $backup, $file;         # reset modification time of edited file  # 
}

## $ok = yes_or_no($prompt);
##   prompt user for yes / no answer
sub yes_or_no {
  my $prompt = shift;
  $prompt = "Are you sure you want to continue ?"
    unless defined $prompt;
  print "$prompt (yes/no) ";
  while (my $answer = <STDIN>) {
    chomp $answer;
    $answer = lc $answer;
    if ("yes" eq $answer) {
      return 1;
    } elsif ("no" eq $answer or "n" eq $answer) {
      return 0;
    } else {
      print "Please answer 'yes' or 'no': ";
    }
  }
}

## $pathname = find_binary($name);
##   find full path to binary with filename $name; returns undef if not found 
sub find_binary {
  my $name = shift;
  my @path = split /:/, $ENV{PATH};
  my $file = undef;
  foreach my $dir (@path) {
    next unless is_absolute_path($dir);		# accept only absolute paths
    if (-f "$dir/$name" and -x "$dir/$name") {
      $file = "$dir/$name";
      last;
    }
  }
  return $file;
}

## $not_ok = has_funny_chars($dir);
##   examine directory or filename for funny characters (' " \ and whitespace)
sub has_funny_chars {
  if ($AllowWhitespace) {
    return scalar $_[0] =~ /[\'\"\\]/;
  }
  else {
    return scalar $_[0] =~ /([\'\"\\]|\s)/;
  }
}

## die_if_funny_chars($dir, ...);
##   examine one or more directories or filenames for funny characters and die if they have some
sub die_if_funny_chars {
  my @problems = grep { has_funny_chars($_) } @_;
  if (@problems) {
    print STDERR "The following ", (@problems > 1) ? "paths contain" : "path contains", 
      " special characters (\' \" \\", ($AllowWhitespace) ? "" : " or whitespace", "),\n";
    print STDERR "which are very likely to confuse some parts of the UCS system:\n";
    foreach my $f (@problems) { print STDERR "\t$f\n"; }
    if ($Force) {
      warn "Installing anyway (--force).  USE AT YOUR OWN RISK!\n";
    }
    else {
      die "Installation aborted.\n";
    }
  }
}

## $string = quote($text);
##   quote $text as literal string that can be inserted into Perl code;
##   ensures that characters ' and \ are not present, and renders an undefined value as undef
sub quote {
  my $text = shift;
  return "undef"
    unless defined $text;
  die "Internal error: can't use '$text' as literal Perl string (because of special characters)\n"
    if $text =~ /[\'\\\n]/;
  return "'$text'";
}

## $ok = is_absolute_path($dir);
##   check that $dir is an absolute path, i.e. it starts with a "/" or a drive letter under Cygwin
sub is_absolute_path {
  my $dir = shift;
  my $ok = ($dir =~ /^\//);
  $ok = ($dir =~ /^[A-Z]:\//)
    if $Cygwin and not $ok;
  return $ok;
}

## $dir = windows_path($dir);
##   under Cygwin, translate $dir to Windows path (inserting Cygwin base directory if necessary);
##   in addition, directory separators are normalised to "/" instead of "\"
sub windows_path {
  my $dir = shift;
  if ($Cygwin) {
    unless ($dir =~ /^[A-Z]:/) {
      die "Internal error: can't translate $dir to Windows path (not an absolute path?)\n"
	unless $dir =~ /^\//;
      $dir = ($dir eq "/") ? $CygwinBase : $CygwinBase.$dir;
    }
    $dir =~ tr[\\][/];
  }
  return $dir;
}
